Package: misty
Type: Package
Title: Miscellaneous Functions 'T. Yanagida'
Version: 0.8.2
Date: 2026-05-16
Authors@R: person(given = "Takuya", family = "Yanagida", email="takuya.yanagida@univie.ac.at", role = c("aut","cre"))
Author: Takuya Yanagida [aut, cre]
Maintainer: Takuya Yanagida <takuya.yanagida@univie.ac.at>
Description: Miscellaneous functions for (1) data handling (e.g., grand-mean and group-mean centering, coding variables and reverse coding items, scale and cluster scores, reading and writing Excel and SPSS files), (2) descriptive statistics (e.g., frequency table, cross tabulation, effect size measures), (3) missing data (e.g., descriptive statistics for missing data, missing data pattern, Little's test of Missing Completely at Random, and auxiliary variable analysis), (4) multilevel data (e.g., multilevel descriptive statistics, within-group and between-group correlation matrix, multilevel confirmatory factor analysis, level-specific fit indices, cross-level measurement equivalence evaluation, multilevel composite reliability, and multilevel R-squared measures), (5) item analysis (e.g., confirmatory factor analysis, coefficient alpha and omega, between-group and longitudinal measurement equivalence evaluation), (6) statistical analysis (e.g., bootstrap confidence intervals, collinearity and residual diagnostics, dominance analysis, between- and within-subject analysis of variance, latent class analysis, t-test, z-test, sample size determination), and (7) functions to interact with 'Blimp' and 'Mplus'.
Depends: R (>= 4.3.0)
License: MIT + file LICENSE
Imports: data.table, ggplot2, haven, lavaan, lme4, rstudioapi
Suggests: boot, clubSandwich, hdf5r, ic.infer, Matrix, mice, mnormt,
        mvnmle, mvtnorm, nleqslv, nlme, patchwork, readxl, robustlmm,
        plyr, writexl
Encoding: UTF-8
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2026-05-16 13:13:41 UTC; takuy
Repository: CRAN
Date/Publication: 2026-05-16 16:00:02 UTC
Built: R 4.6.0; ; 2026-05-16 17:23:41 UTC; unix
