# Generated by roxygen2: do not edit by hand
S3method(ci.mean.diff,default)
S3method(ci.mean.diff,formula)
S3method(ci.prop.diff,default)
S3method(ci.prop.diff,formula)
S3method(cohens.d,default)
S3method(cohens.d,formula)
S3method(test.t,default)
S3method(test.t,formula)
S3method(test.z,default)
S3method(test.z,formula)
S3method(plot,misty.object)
S3method(print,misty.object)
export(aov.b)
export(aov.w)
export(as.na)
export(blimp.bayes)
export(blimp.plot)
export(blimp.print)
export(blimp)
export(blimp.run)
export(blimp.update)
export(boot.bs)
export(center)
export(cfa.satcor)
export(check.collin)
export(check.outlier)
export(check.resid)
export(chr.color)
export(chr.grep)
export(chr.grepl)
export(chr.gsub)
export(chr.omit)
export(chr.trim)
export(chr.trunc)
export(ci.cor)
export(ci.mean)
export(ci.mean.w)
export(ci.mean.diff)
export(ci.median)
export(ci.prop)
export(ci.prop.diff)
export(ci.sd)
export(ci.var)
export(clear)
export(cluster.rwg)
export(cluster.scores)
export(coding)
export(coeff.robust)
export(coeff.std)
export(cohens.d)
export(cor.matrix)
export(crosstab)
export(descript)
export(df.check)
export(df.duplicated)
export(df.head)
export(df.long)
export(df.merge)
export(df.move)
export(df.rbind)
export(df.rename)
export(df.sort)
export(df.unique)
export(df.subset)
export(df.tail)
export(df.wide)
export(difftest.chibarsq)
export(dominance.manual)
export(dominance)
export(effsize)
export(freq)
export(growth.satcor)
export(indirect)
export(item.alpha)
export(item.cfa)
export(item.dfi)
export(item.invar)
export(item.noninvar)
export(item.omega)
export(item.reverse)
export(item.scores)
export(kurtosis)
export(lagged)
export(lavaan.satcor)
export(libraries)
export(modcomp)
export(mplus.bayes)
export(mplus.lca)
export(mplus.lca.summa)
export(mplus.plot)
export(mplus.print)
export(mplus)
export(mplus.run)
export(mplus.update)
export(multilevel.cfa)
export(multilevel.cor)
export(multilevel.descript)
export(multilevel.fit)
export(multilevel.icc)
export(multilevel.indirect)
export(multilevel.invar)
export(multilevel.omega)
export(multilevel.r2.manual)
export(multilevel.r2)
export(na.as)
export(na.auxiliary)
export(na.coverage)
export(na.descript)
export(na.indicator)
export(na.pattern)
export(na.prop)
export(na.satcor)
export(na.test)
export(read.data)
export(read.dta)
export(read.mplus)
export(read.sav)
export(read.xlsx)
export(rec)
export(restart)
export(robust.lmer)
export(script.copy)
export(script.close)
export(script.new)
export(script.open)
export(script.save)
export(sem.satcor)
export(setsource)
export(sim.lavaan)
export(size.cor)
export(size.mean)
export(size.prop)
export(skewness)
export(summa)
export(test.levene)
export(test.t)
export(test.welch)
export(test.z)
export(uniq)
export(uniq.n)
export(write.data)
export(write.data1)
export(write.dta)
export(write.mplus)
export(write.result)
export(write.sav)
export(write.xlsx)
importFrom("stats", "AIC","aov", "anova", "as.formula", "BIC", "chisq.test", "contrasts", "contr.helmert",
           "contr.poly", "contr.sum", "contr.treatment", "coef", "coefficients",
           "cooks.distance", "confint", "cor", "cor.test", "cov", "cov2cor", "density", "df",
           "dfbetas", "df.residual", "dnorm", "dt", "fft", "formula", "hatvalues",
           "integrate", "IQR", "lag", "lm", "loess", "logLik", "mad", "mahalanobis", "median",
           "model.frame", "model.matrix", "na.omit", "nextn", "nobs", "oneway.test", "optim",
           "optimize", "pnorm", "predict", "resid", "residuals", "rnorm", "rstandard", "rstudent", 
           "runif", "setNames", "sigma", "summary.lm", "pchisq", "qbeta", "qbinom", "qchisq", "qf", 
           "qnorm", "qt", "quantile", "reshape", "sd", "terms", "time", "t.test", "p.adjust", "pf", 
           "pt", "uniroot", "update", "var", "vcov", "weights")
importFrom("grDevices", "hcl.colors", "hcl.pals")
importFrom("utils", "as.relistable", "available.packages", "combn", "head",
           "install.packages", "installed.packages", "packageDescription", "read.csv",
           "read.table", "relist", "setTxtProgressBar", "txtProgressBar", "write.csv", "write.csv2", "write.table")
